Widgets at CMS page content
===========================

**Note:**
For now widgets are only supported by CMS page content but you can add support to any text variables 
(see [How to support widgets in my content](#how-to-support-widgets-in-custom-content)).
 
With widgets you can render any html text generated by your custom PHP code, like rendering marketing banner or contact form.

To register new widget you should add class that extends `Oro\Bundle\CMSBundle\Widget\WidgetInterface` 
and implement `render` method that apply as argument widget options and return string. 

Options should be simple scalar values that can be easily added by administrator.


You should register every widget at the service container with `oro_cms.widget` tag, each tag should provide user friendly `alias` that will be used by administrator for referencing to widgets during managing content that supports widgets.
 
**Example:**

 ````yml
 # AppBundle/Resources/config/services.yml
 
 acme_demo.contact_us_form:
     class: 'AppBundle\Widget\ContactFormWidget'
     lazy: true
     tags:
         - {name: oro_cms.widget, alias: contact_form}
 ````
 
 Then administrator can use above widget in supported blocks with placeholder `{{ widget('contact_form', {foo: 'bar'}) }}`, where `foo: bar` is an option with value that will be available in `WidgetInterface::render()` method 

**Notice:**
It's better to mark widgets as `lazy: true` to load them only on pages where they are used.

How to support widgets in custom content
----------------------------------------
To support widgets in any text you should apply `render_content` filter to it at twig template.
 
 For example:
 
  ````twig
  {{ entity.content | render_content }}
  ````
